/* ClientSocket.cpp
 * Author: Jeroen van Montfort
 * 
 * +------------Oo.------+
 * | TRANSPORT EMPIRE -> |
 * +---------------------+
 * 
 * 
 */

#include "ClientSocket.h"
#include <iostream>
#ifdef WIN32
#include <winsock2.h>
#else
#include <sys/socket.h>
#include <arpa/inet.h>
#endif
#include <unistd.h>

using std::cerr;
using std::endl;

// Static variables
std::string ClientSocket::Author = "Hellfire";
std::string ClientSocket::Name = "ClientSocket";
std::string ClientSocket::Abbreviation = "CLSC";
long ClientSocket::Count = 0;


// Constructor
ClientSocket::ClientSocket() {
  // Initialize Winsock2 on Windows platforms
  #ifdef WIN32
  if(Count == 0) {
    WSADATA wsaData;
    int err = WSAStartup(0x0002, &wsaData);
    if(err) {
      cerr << "Error starting Winsock2" << endl;
    }
  }
  #endif
  
  Count++;
  Sock = 0;
}

// Destructor
ClientSocket::~ClientSocket() {
  Count--;
  
  if(Sock != 0) {
    close(Sock);
  }
  
  // Deinitialize Winsock2 on Windows platforms
  #ifdef WIN32
  WSACleanup();
  #endif
}

// Connect to a server
bool ClientSocket::Connect(std::string hostname, unsigned short port) {
  cerr << "Connecting to " << hostname << ":" << port << endl;
  
  int err;
  
  // Open a socket
  Sock = socket(PF_INET, SOCK_STREAM, 0);

  if(!Sock) {
    cerr << "Error while opening a socket!" <<endl;
    return false;
  }
  
  // Step 2: Connect the socket to the hostname and address
  struct sockaddr_in addr;
  memset(&addr, 0, sizeof(addr));
  addr.sin_family = AF_INET;
  addr.sin_addr.s_addr = inet_addr("127.0.0.1");
  addr.sin_port = htons(port);
  
  err = connect(Sock, (struct sockaddr*)&addr, sizeof(addr));
  if(err) {
    cerr << "Error while connecting the socket to an address!" << endl;
    return false;
  }
  
  return true;
}

// Send a packet to the server
bool ClientSocket::SendPacket(Packet *packet) {
  char *buffer;
  unsigned long bufLen = packet->Length();
  
  buffer = new char[bufLen];
  packet->FillBuffer(buffer, bufLen);
  
  send(Sock, buffer, bufLen, 0);
  
  delete buffer;
  
  return true;
}
