/* ClientSocket.cpp
 * Author: Jeroen van Montfort
 * 
 * +------------Oo.------+
 * | TRANSPORT EMPIRE -> |
 * +---------------------+
 * 
 * 
 */

#include "ServerSocket.h"
#include "TestPacket.h"

#include <iostream>
#ifdef WIN32
#include <winsock2.h>
#else
#include <sys/socket.h>
#include <arpa/inet.h>
#endif
#include <unistd.h>

using std::cerr;
using std::endl;

// Static variables
std::string ServerSocket::Author = "Hellfire";
std::string ServerSocket::Name = "ClientSocket";
std::string ServerSocket::Abbreviation = "CLSC";
long ServerSocket::Count = 0;


// Constructor
ServerSocket::ServerSocket() {
  // Initialize Winsock2 on Windows platforms
  #ifdef WIN32
  if(Count == 0) {
    WSADATA wsaData;
    int err = WSAStartup(0x0002, &wsaData);
    if(err) {
      cerr << "Error starting Winsock2" << endl;
    }
  }
  #endif

  Count++;
}

// Destructor
ServerSocket::~ServerSocket() {
  Count--;
  
  if(Sock != 0) {
    close(Sock);
  }
  
  // Deinitialize Winsock2 on Windows platforms
  #ifdef WIN32
  WSACleanup();
  #endif
}

// Open a port
bool ServerSocket::OpenPort(unsigned short port) {
  int err;
  
  // Open a socket
  Sock = socket(PF_INET, SOCK_STREAM, 0);

  if(!Sock) {
    cerr << "Error while opening a socket!" <<endl;
    return false;
  }
  
  // Bind the socket to 0.0.0.0:port
  struct sockaddr_in addr;
  memset(&addr, 0, sizeof(addr));
  addr.sin_family = AF_INET;
  addr.sin_addr.s_addr = 0;
  addr.sin_port = htons(port);
  
  err = bind(Sock, (struct sockaddr*)&addr, sizeof(addr));
  if(err) {
    cerr << "Error while binding the socket to an address!" << endl;
    close(Sock);
    return false;
  }
  
  // Set connection queue length
  err = listen(Sock, 10);
  if(err) {
    cerr << "Error while setting queue length!" << endl;
    close(Sock);
    return false;
  }

  return true;
}

// Accept a connection
ServerSocket* ServerSocket::AcceptConnection() {
  // Accept a connection
  int newSock = accept(Sock, NULL, 0);
  
  if(newSock <= 0) {
    return NULL;
  }
  
  ServerSocket *newSocket = new ServerSocket();
  newSocket->Sock = newSock;
  
  return newSocket;
}

// Receive a packet
Packet* ServerSocket::ReceivePacket() {
  char *buffer = new char[4];
  
  int bytes = recv(Sock, buffer, 4, MSG_PEEK);
  
  if(bytes != 4) {
    delete buffer;
    return NULL;
  }
  
  unsigned short type = buffer[0] << 8 | buffer[1];
  unsigned short length = buffer[2] << 8 | buffer[3];
  
  if(4 < length) {
    delete buffer;
    buffer = new char[length];
    bytes = recv(Sock, buffer, length, 0);
  }
  
  Packet *packet;
  // This switch statement should move to the PacketFactory class.
  switch(type) {
    case 1:
      packet = new TestPacket();
      break;
    default:
      delete buffer;
      return NULL;
  }
  
  // Clean up
  delete buffer;
  
  return packet;
}
